﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Algorithms.NCA.Views {
  [View("NCA Model View")]
  [Content(typeof(NCAModel), IsDefaultView = true)]
  public partial class NCAModelView : ItemView {
    private ViewHost viewHost = new ViewHost();
    private ScatterPlot scatterPlot = new ScatterPlot();

    public new NCAModel Content {
      get { return (NCAModel)base.Content; }
      set { base.Content = value; }
    }

    public NCAModelView() {
      InitializeComponent();
      viewHost.Visible = false;
      viewHost.Dock = DockStyle.Fill;
      Controls.Add(viewHost);
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers on the Content here
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers on the Content here
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        scatterPlot.Rows.Clear();
        messageLabel.Visible = false;
        viewHost.Visible = true;
      } else {
        var matrix = Content.TransformedTrainingset;
        if (matrix.GetLength(1) != 2) {
          messageLabel.Visible = true;
          viewHost.Visible = false;
        } else {
          messageLabel.Visible = false;
          viewHost.Content = null;
          viewHost.Visible = true;
          UpdateScatterPlot(matrix);
          viewHost.Content = scatterPlot;
        }
      }
    }

    private void UpdateScatterPlot(double[,] matrix) {
      scatterPlot.Rows.Clear();
      var classes = Content.ClassValues;
      var rows = new Dictionary<double, ScatterPlotDataRow>();
      for (int i = 0; i < matrix.GetLength(0); i++) {
        if (!rows.ContainsKey(classes[i])) rows[classes[i]] = new ScatterPlotDataRow("Class " + classes[i].ToString(), string.Empty, Enumerable.Empty<Point2D<double>>());
        rows[classes[i]].Points.Add(new Point2D<double>(matrix[i, 0], matrix[i, 1]));
      }
      scatterPlot.Rows.AddRange(rows.Values);
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
    }

    #region Event Handlers
    // TODO: Put event handlers here
    #endregion
  }
}
